//
//  main.swift
//  GameBattleshipTerminal
//
//  Created by Piotr Fulmański on 2020.02.26.
//  Copyright © 2020 Piotr Fulmański. All rights reserved.
//

import Foundation

func getIntFromCommandLine(message: String, rangeMin: Int, rangeMax: Int) -> Int {
    print(message)
    while(true) {
        if let input = readLine()
        {
            if let int = Int(input)
            {
                if int >= rangeMin && int <= rangeMax {
                    return int
                } else {
                    print("\(input) is not in range [\(rangeMin)-\(rangeMax)]. Please try againt")
                }
            }
            else{
                print("\(input) is not a valid integer. Please try againt")
            }
        }
    }
}

func getShotCoordinatesForPlayer(engine: EngineGameBattleship,
                                  maxRows: Int,
                                  maxCols: Int) -> (row: Int, col: Int)? {
    while(true) {
        let row = getIntFromCommandLine(message: "Enter row",
                                        rangeMin: 1,
                                        rangeMax: maxRows)
        let col = getIntFromCommandLine(message: "Enter column",
                                        rangeMin: 1,
                                        rangeMax: maxCols)
        
        if engine.mayShot(who: .player, row: row, col: col) {
            return (row: row, col: col)
        }
    }
}

let shipsSize = [4, 3, 3, 2, 2, 2, 1, 1, 1, 1]
let game = EngineGameBattleship(shipsSize: shipsSize)
let opponentReady = game.shipsAutoSetup(shipsSize: shipsSize,
                                        maxTriesPerShip: 20,
                                        who: .opponent)
let playerReady = game.shipsAutoSetup(shipsSize: shipsSize,
                                      maxTriesPerShip: 20,
                                      who: .player)
var whoseTurn = EngineGameBattleship.Who.player
var coordinates: (row: Int, col: Int)?

game.printBoards()

if opponentReady, playerReady {
    while(true){
        print("\n\n\n TURN")
        print(whoseTurn)
        
        if whoseTurn == EngineGameBattleship.Who.player {
            coordinates = getShotCoordinatesForPlayer(engine: game, maxRows: 10, maxCols: 10)
            if coordinates == nil {
                print("Player can't shoot")
                break
            } else {
                print("Shot at row \(coordinates!.row) and column \(coordinates!.col)")
            }
            game.shot(who: whoseTurn, row: coordinates!.row, col: coordinates!.col)
            whoseTurn = EngineGameBattleship.Who.opponent
        } else {
            coordinates = game.getShotCoordinatesForOpponent(maxTries: 100)
            if coordinates == nil {
                print("Opponent can't shoot")
                break
            } else {
                print("Shot at row \(coordinates!.row) and column \(coordinates!.col)")
           }
            game.shot(who: whoseTurn, row: coordinates!.row, col: coordinates!.col)
            whoseTurn = EngineGameBattleship.Who.player
        }
        
        game.printBoards()
        if let who = game.checkWhoWins() {
            print("The winner is \(who)")
            break
        }
    }
} else {
    print("Can't arrange all ships")
}



